#!/usr/bin/env python
"""
    gladex - Generate source code from Glade XML files.

    Copyright (C) 2007 See AUTHORS list for copyright holders

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import commands
import time
#from threading import Timer
from threading import *
from time import sleep
import sys, os, stat
import pwd, grp, locale
import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade
import inspect
import gladex_functions as gxf
import plugins

global_data = {}
global_data["main_window_mapped"]=False

class fallbackplugin:
	def info(self):
		pass
	info.name="Fallback"
	info.version=1
	info.compatibility=0.4
	info.description="This is a fallback plugin"
	info.plugin_type="generator"


	def widget(self):
		global plugdir
		return gtk.Label("Error with plugin!")
	#

	def execute():
		return "Error with Plugin!"
	#


	def on_glade_file_change(widget):
		pass
	#

	def on_directory_change(widget):
		pass
#


def load_plugins():
	"""load_plugins()
	this will load all the plugins found by inspect
	if error happens on load, a fallbackplugin gets loaded
	Returns list of loaded plugins.
	"""
	plugins_list = []
	plugs = dir(plugins)
	for i in range(len(plugs)):
		if plugs[i][0:2]=="__":continue
		try:
			plugins_list.append(__import__("plugins/"+inspect.getmodulename(plugs[i]+".py")))
			t=plugins_list[-1].widget
			t=plugins_list[-1].execute
			t=plugins_list[-1].info
			t=plugins_list[-1].on_directory_change
			t=plugins_list[-1].on_glade_file_change
		except:
			plugins_list.append(fallbackplugin())
	return plugins_list




def gtk_main_quit(widget, data, wtree):
	"""gtk_main_quit( ... )
	This functions destroys all threads and quits gtk"""
	a = enumerate()
	for x in a: 
		#print dir(x)
		try:
			x._Thread__stop()
		except:
			pass
	gtk.main_quit()
	pass

def on_load_example_activate(widget, data, wtree):
	"""on_load_example_activate( ... )
	This is a event function, called when 'example' is clicked from the menu.
	"""
	wtree.get_widget("output_directory_chooser").set_filename(global_data["home"]+"/Desktop")
	glade_file_chooser = global_data["tree"].get_widget("glade_file_chooser")
	glade_file_chooser.set_filename(os.path.split(__file__)[0]+"/examples/example.glade")
	pass


def on_MainWindow_map_event(widget, data, wtree):
	"""on_MainWindow_map_event(...)
	This is a event functions which is run when the program is first run.
	I used a map signal, but in future version it will use a init function
	called from the main py program. Check the output of the python plugin
	and you will see what I mean.
	"""
	global global_data
	if global_data["main_window_mapped"]==True:
		return
	global_data["main_window_mapped"] = True
	global_data["plugins"] = []
	plugin_list = load_plugins()
	
	global_data["tree"] = wtree
	
	plugin_list_combo = wtree.get_widget("plugin_list")
	
	d = os.getcwd()
	global_data["threadrunning"] = False
	home = global_data["home"]=commands.getoutput("cd ~;pwd")
	wtree.get_widget("glade_file_chooser").set_current_folder_uri(home)
	wtree.get_widget("output_directory_chooser").set_current_folder_uri(home)
	commands.getoutput("cd "+d)
	
	listStore = gtk.ListStore(str)
	listStore.append(["Select Plugin ..."])
	for plug in plugin_list:
		global_data["plugins"].append(plug)
		#print plug.info.name
		plugin_list_combo.append_text(plug.info.name)
		listStore.append([plug.info.name])
		#plugin_list_combo.append_text("hello")
	
	
	#plugin_list_combo.set_model(listStore)
	print plugin_list_combo.get_model()
	
	plugin_list_combo.set_active(0)
	
	glade_file_filter = gtk.FileFilter()
	glade_file_filter.add_mime_type("glade")
	glade_file_filter.add_pattern("*.glade")
	wtree.get_widget("glade_file_chooser").set_filter(glade_file_filter)
	pass
#
def send_status_message(messages):
	"""send_status_message(messages)
	Uses threading to help show_message send a message
	to the status bar.
	
	messages - string of text to send
	"""
	bar = global_data["tree"].get_widget("statusbar")
	global_data["threadrunning"]=True
	for x in messages:
		id = bar.get_context_id(x)
		bar.push(id, x)
		sleep(2)
		bar.pop(id)
	global_data["threadrunning"]=False
#

def show_message(msg, status_message=False):
	"""show_message(msg, status_message=False)
	msg - message to send
	status_message - if False then message is dialog, if True, then status bar
	"""
	global global_data
	if status_message:
		msg = msg.replace("\t","   ")
		if global_data["threadrunning"] == False:
			t = Timer(0, send_status_message, [msg.split("\n")])
			t.start()
		return
	message_window = global_data["tree"].get_widget("message_window")
	message_window.set_markup(msg)
	responce = message_window.run()
	if responce == gtk.RESPONSE_OK or responce==gtk.RESPONSE_DELETE_EVENT:
		message_window.hide()
#

def on_plugin_list_changed(widget, data, wtree):
	"""on_plugin_list_changed( ... )
	This is a event function, called when the plugin list changes.
	"""
	global global_data
	#print "plugin list changed"
	load_box = wtree.get_widget("load_box")
	for x in load_box.get_children():
		load_box.remove(x)
	activate_plugin_index = widget.get_active() -1 # minus 1 because 0 is 'select plugin ...' in the combo list
	#print activate_plugin_index
	if activate_plugin_index < 0: return
	newWidget = global_data["plugins"][activate_plugin_index].widget()
	newWidget.unparent()
	load_box.add(newWidget)
	newWidget.show_all()
	pass



def on_info_button_clicked(widget, data, wtree):
	"""on_info_button_clicked( ... )
	This is a event function, called when the info button is clicked.
	"""
	plug = get_selected_plugin()
	if plug == None:
		message = "Select a plugin"
	else:
		message = plug.info.description
	show_message(message)


def get_selected_plugin():
	"""get_selected_plugin()
	Returns currently selected plugin from plugin_list widget.
	if not plugin is selected, Return None
	"""
	global global_data
	plugin_combo = global_data["tree"].get_widget("plugin_list")
	activate_plugin_index = plugin_combo.get_active() - 1
	if activate_plugin_index < 0:
		return None
	else:
		return global_data["plugins"][activate_plugin_index]
#

def get_mode_info(mode):
	"""get_mode_info(mode)
	mode - the mode number, like 0777
	Returns info from mode file number.
	"""
	aperms = {"USR":[], "GRP":[], "OTH":[]}
	mode=stat.S_IMODE(mode)
	for who in ["USR", "GRP", "OTH"]:
		for what in ["R", "W", "X"]:
			if mode & getattr(stat,"S_I"+what+who):
				aperms[who].append(what.lower())
	return aperms
#

def get_permission_info(path):
	"""get_permission_info(path)
	path - the path to check permissions on
	Returns permissions and ownership information about a file.
	"""
	if not os.path.exists(path):
	        return False
	try:
		stat_info=os.lstat(path)
	except:
		return False
	
	perms, colour, link = get_mode_info(stat_info.st_mode)

	try:
		name = pwd.getpwuid(stat_info.st_uid)[0]
	except KeyError:
		name = stat_info.st_uid
	try:
		group = grp.getgrgid(stat_info.st_gid)[0]
	except KeyError:
		group = stat_info.st_gid
	info = get_mode_info(stat_info.st_mode)
	return [name, group, info]
#


def has_permissions(path):
	"""has_permissions(path)
	This functions ties in with the other permissions function
	path - the path to check permissions on
	Returns True if the user can write and read
	False otherwise
	"""
	info = get_permission_info(path)
	if info == False:
		show_message("ERROR")
		return False
	currentName = commands.getoutput("whoami")
	if currentName == info[0] and ["r","w","x"] == info[2]["USR"]:
		return True
	elif currentName == info[2] and ["r","w", "x"] == info[2]["GRP"]:
		return True
	elif ["r","w", "x"] == info[2]["OTH"]:
		return True
	else:
		return False
#

def on_execute_clicked(widget, data, wtree):
	"""on_execute_clicked( ... )
	This is a event function, called when the execute button is clicked.
	"""
	global global_data
	## TODO:
	# check if a glade file and a plugin is chosen
	c1 = wtree.get_widget("output_directory_chooser").get_filename()
	
	c2 = wtree.get_widget("glade_file_chooser").get_filename()
	if None in [c1, c2]:
		show_message("Choose a glade file",True)
		return
	
	#check permissions
	if not has_permissions(c1):
		show_message("ERROR: Insufficient permission.\n Change output directory.",False)
		return
	
	# calls execute function from plugin
	plug = get_selected_plugin()
	if plug == None:
		message = "Select a plugin"
	else:
		#execute plugin code
		message = plug.execute()
	# display message
	show_message(message, True)
	pass
#

def on_about_menu_activate(widget, data, wtree):
	"""on_about_menu_activate( ... )
	This is a event function, called when about is clicked from the menu.
	The function will open and handel the about dialog.
	"""
	about = wtree.get_widget("about_gladex")
	responce = about.run()
	if responce == gtk.RESPONSE_CANCEL or responce==gtk.RESPONSE_DELETE_EVENT:
		about.hide()
	pass
#

def on_help_menu_activate(widget, data, wtree):
	"""on_help_menu_activate( ... )
	This is a event function, that is called when the 'help' is clicked in the menu.
	uses yelp upon our docs.
	"""
	yelp = os.popen("yelp file://"+os.path.dirname(__file__)+"/docs/gladex-manual.xml")
	os.unlink(yelp)
#

def on_glade_file_chooser_selection_changed(widget, data, wtree):
	"""on_glade_file_chooser_selection_changed( ... )
	This is a event function, called when the file chooser is changed.
	"""
	gladefile = widget.get_filename()
	if gladefile is not None and gladefile.find(".glade") < 0:
		show_message("This is not a glade file")
		widget.set_filename("")
	
	if global_data["home"] == global_data["tree"].get_widget("output_directory_chooser").get_filename() and gladefile!=None:
		f = gladefile[:gladefile.rfind("/")]
		global_data["tree"].get_widget("output_directory_chooser").set_filename(f)
		
	plug = get_selected_plugin()
	if plug == None: return None
	else:
		try:
			plug.on_glade_file_change(widget)
		except:
			return None
#

def on_output_directory_chooser_selection_changed(widget, data, wtree):
	"""on_output_directory_chooser_selection_changed( ... )
	This is a event function, called when the directory choser is changed
	"""
	plug = get_selected_plugin()
	if plug == None: return None
	else:
		try:
			plug.on_directory_change(widget)
		except:
			return None
	pass

def open_file_directory(widget, data, wtree):
	name = widget.get_property("name")
	directory = ""
	if name == "open_glade_file_directory":
		tmp = global_data["tree"].get_widget("glade_file_chooser").get_filename()
		if tmp == None or tmp == "None":
			show_message("Choose a glade file", True)
			return
		tmp = tmp[:tmp.rfind("/")]
		directory = tmp
	elif name == "open_output_directory_button":
		directory = global_data["tree"].get_widget("output_directory_chooser").get_filename()
	else: return
	commands.getoutput("nautilus "+directory)
	pass
#	
def on_open_examples_activate(widget,data,wtree):
	commands.getoutput("nautilus "+os.path.dirname(__file__)+"/examples")
	pass
#
