"""
    gladex - Generate source code from Glade XML files.

    Copyright (C) 2007 See AUTHORS list for copyright holders

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
#import py_compile
#py_compile.compile("gladex_callbacks.py")
import gladex_callbacks as gxc
#from gladex_callbacks import *
import os
from xml.dom.minidom import parse as pxml

reload(gxc)
#print dir(gxc)
show_message = gxc.show_message

def get_gladex_tree():
	""" get_gladex_tree( ... )
	Returns gladeXML tree.
	"""
	return gxc.global_data["tree"]
# the save global variable is used in get_handlers so that the glade file does not
# need to be loaded twice
save = {}

def get_handlers():
	"""
	Inputs a Glade file name. Outputs a list of handlers in that Glade file.
	"""
	dom = pxml(gxc.global_data["tree"].get_widget("glade_file_chooser").get_filename())
	s = dom.getElementsByTagName("signal")
	handlers={}
	for x in s: handlers[x.getAttribute("handler")]=''
	return handlers.keys()
#


def validate_mode(mode):
	"""validate_mode(mode)
	mode - string representation of file mode
 r open file for reading only, place pointer at 0 
 w open file for writing, truncates file, or creates a new one
 a open file for appending, place pointer at end, create new file if not exist
 b option for binary files
 + reading and writing, according to rules
	"""
	m = "rwab+"
	for x in mode:
		if x not in m: return False
	return True
	
class Gxio:
	"""
	Gxio(inputfile, outputfile)
	inputfile - a File object. see File in this module. For input
	inputfile - a File object. see File in this module. For output
	
	Gxio is designed to make file manipulation simple. One of the main
	reason for Gxio is the replace function which replaces key terms
	with the corresponding key values.
	"""
	def __init__(self, inputfile, outputfile):
		self.input_file_text = inputfile.read_text()
		self.output_file = outputfile
		self.output_file_text = self.input_file_text
	def set_text(self, text):
		"""
		set_text(text)
		text - input of text
		sets text of output file
		"""
		self.output_file_text = text
	def get_text(self):
		"""
		Returns text of output file
		"""
		return self.output_file_text
	def replace(self, dic): 
		"""replace(dic)
	dic - the dictionary of keys and terms to replace
	
	goes through the dictionary and replaces keys with the values, when
	in the text of the output file.
		"""
		for key in dic.keys():
			self.output_file_text = self.output_file_text.replace(key, dic[key])
	def write_out(self, outperm=0644):
		"""write_out(outperm=0644)
		outperm - a octal form of mode for files. for example: 0644.
		will commit the text of the output file to a actual file, with the
		given permissions.
		"""
		self.output_file.write_text(self.output_file_text)
		self.output_file.close()
		os.chmod(self.output_file.uri, outperm)
#


class File(file):
	"""
	File(file) - class
	file - base class of File
	File(filename, filemode=None)
	filename - string location of file 
	filemode - open mode. Like, "r" or "w+"
	
	the File object is based on file, can can do the same functions as file.
	File extends file is such a way where you can get values like file
	extension or file name or path.
	"""
	def __init__(self,filename, filemode=None):
		"""
		filename is name of file
		filemode is mode to open
		if file mode is None, then analyze only
		"""
		self.filemode = filemode
		if self.filemode!=None:
			super(File,self).__init__(filename, self.filemode)
		self.input_file = ""
		self.input_text = ""
		
		#self.uri = self.name
		self.uri = filename
		pane = os.path.split(self.uri)
		self.path = pane[0]+"/"
		ne = pane[1].split(".")
		ne.append('')
		self.title = ne[0]
		self.ext = ne[1]
		#print "################# ",self.uri
		self.title_ext = self.title + "." + self.ext
	def read_text(self):
		"""
		Returns text of file
		"""
		if self.filemode==None:return None
		if self.input_text != "": return self.input_text
		self.input_text =  self.read()
		return self.input_text
	def write_text(self, text):
		"""
		Writes text to file
		"""
		if self.filemode==None:return None
		self.write(text)
#
def file_copy(source, path_target):
	"""file_copy(source, path_target)
	source - ex: /asd/asdf/my.glade
	path_target - ex: /asd/asdf/
	
	This function will copy a file from source to target.
	"""
	input = source
	#if '/' in source:
	source = source[source.rfind('/')+1:]
	if os.path.exists(path_target+source):
		return
	a = open(input, "r")
	b = open(path_target+source,"w")
	b.write(a.read())
	b.close()
	a.close()

def debug(message, level=1, pause=False):
	"""
	debug(message, level=1, pause=False)
	debug message function. just prints it out message.
	give the ability to turn off print, instead of commenting out
	level 1, general information to user
	level 2, more verbose information to user
	level 3, development information to programmer
	if pause is True, then the output will pause.
	"""
	if not debug.debug: return
	#D.level is is defined outside of this function.
	if level in debug.levels:
		debug.hold += message+"\n"
		print message
	if pause:
		raw_input("\nPRESS ENTER")
debug.debug = True # option to overide everything and show no messages
debug.levels = [1] # the level that you want to print
debug.hold = "" #holder for all messages


def exit(error):
	"""
	exit with error message
	"""
	print error
	sys.exit()
#
