#!/usr/bin/env python
"""
    gladex - Generate source code from Glade XML files.

    Copyright (C) 2007 See AUTHORS list for copyright holders

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
sys.path.insert(0,"../..")
import gladex_functions as gxf
import gtk
import os
import re
import string
import inspect

## Global variables ##
main_widget = None
#plugdir holds the path to this file
plugdir =os.path.split(__file__)[0]
tree = None
widgets = {}

def info():
	pass
info.name="Python Code Generator"
info.version=1
info.compatibility=0.4
info.description="""Python Code Generator
	This is the code generator for python, using pygtk.
	Ubuntu, and other GNOME driven distributions, will
	have the required libglade librarys needed to run
	the python code.
	"""
info.plugin_type="generator"


def widget():
	"""
	This function runs when the plugin is loaded.
	Returns GTK widget of what to load.
	"""
	global main_widget, tree, parent_data, widgets,plugdir
	#parent_data = parent_input
	
	if main_widget == None:
	        gladefile = plugdir+'/python.glade'
	        tree = gtk.glade.XML(gladefile)
	        main_widget = tree.get_widget("main_widget")
	        dic ={"on_info_button_clicked":on_info_button_clicked}
	        tree.signal_autoconnect(dic)
	
	widgets["callbacks_file_name"] = tree.get_widget("callbacks_file_name_entry")
	widgets["python_file_name"] = tree.get_widget("python_file_name_entry")
	widgets["class_name"] = tree.get_widget("python_class_name_entry")
	widgets["style"] = tree.get_widget("code_style")
	widgets["style"].set_active(0)
	widgets["indentation"] = tree.get_widget("indentation")
	widgets["indentation"].set_active(0)
	widgets["glade_file_entry"] = gxf.get_gladex_tree().get_widget("glade_file_chooser")
	
	update_entry_feilds()
	
	return main_widget
#

def make_valid(text, extra=""):
	"""
	This function is executed when the plugin is loaded.
	Returns GTK widget of what to load.
	"""
	valid = "abcdefghijklmnopqrstuvwxyz"
	valid += valid.upper() + "._" + extra
	r=""
	for x in text:
		if x in valid:
			r+=x
	return r
#


def update_entry_feilds():
	"""
	This function is executed when the fields of the main widget
	need to be updated.
	"""
	glade_file_name = widgets["glade_file_entry"].get_filename()
	if glade_file_name == None: glade_file_name="sample"
	else:
		glade_file_name = glade_file_name[glade_file_name.rfind("/")+1:-6]
	
	callname = glade_file_name+"_callbacks.py"
	widgets["callbacks_file_name"].set_text(make_valid(callname))
	
	classname = glade_file_name[0].upper()+glade_file_name[1:]+"Class"
	widgets["class_name"].set_text(make_valid(classname))
	
	pythonname = glade_file_name+".py"
	widgets["python_file_name"].set_text(make_valid(pythonname))
#


def execute():
	"""
	This function is executed when the execute button is pressed from
	the main window.
	"""
	valid = "._abcdefghijklmnopqrstuvwxyz"
	valid += valid.upper()
	
	## check in all feilds
	gxt = gxf.get_gladex_tree()
	working_dir = gxt.get_widget("output_directory_chooser").get_filename()
	glade_file_name = gxt.get_widget("glade_file_chooser").get_filename()
	callbacks_file_name = tree.get_widget("callbacks_file_name_entry").get_text()
	python_file_name = tree.get_widget("python_file_name_entry").get_text()
	class_name = tree.get_widget("python_class_name_entry").get_text()
	style = tree.get_widget("code_style").get_active_text()
	indentation = tree.get_widget("indentation").get_active_text()
	
	
	glade_file = gxf.File(glade_file_name)
	callbacks_file = gxf.File(callbacks_file_name)
	python_file = gxf.File(python_file_name)
	
	## validate feilds
	if glade_file.ext != "glade":
		return "glade file must end in .glade"
	
	if callbacks_file.ext != "py":
		return "callbacks file must end in .py"
	
	if python_file.ext != "py":
		return "python file must end in .py"
	
	
	for x in class_name:
		if x not in valid:
			return """there is a invalid character in class name
please only use letters/Caps or underscores"""
	for x in callbacks_file_name:
		if x not in valid:
			return """there is a invalid character in callbacks file name
please only use letters/Caps or underscores"""
	for x in python_file_name:
		if x not in valid:
			return """there is a invalid character in python file name
please only use letters/Caps or underscores"""
					
	if indentation == "" or style == "":
			return "please select a indentation and style type"
	if indentation == "tab":
		indent = "\t"
	elif indentation == "4 spaces":
		indent = "    "
	else:
		indent = "\t"
	
	## get callbacks functions
	callbacks = gxf.get_handlers()
	
	### style classic #####################
	if style == "Minimal":
		styledir = plugdir+"/styles/classic/"
	### style no regeneration #############
	if style == "Full": 
		styledir = plugdir+"/styles/no-rewrite/"
	nmbs="0123456789"
	replace = {
		"{I}":indent,
		"{CLASS_NAME}":class_name,
		"{DICTIONARY}":get_dictionary(indent),
		"{GLADE_FILE_TITLE_EXT}":glade_file.title_ext,
		"{GLADE_FILE_TITLE}":make_valid(glade_file.title,nmbs+"_"),
		"{CALLBACKS_FILE_TITLE_EXT}":callbacks_file.title_ext,
		"{CALLBACKS_FILE_TITLE}":make_valid(callbacks_file.title,nmbs+"_")
		}
	
	#input files
	#glade file copy
	##glade_file = gxf.File(glade_file_name, "r")
	##glade_file_out = gxf.File(working_dir+"/"+glade_file.title_ext, "w")
	##gfo = gxf.Gxio(glade_file, glade_file_out)
	##gfo.write_out()
	##glade_file.close()
	##glade_file_out.close()
	gxf.file_copy(glade_file_name, working_dir+"/")
	
	outputdir = working_dir+"/"
	#python file writeout useing template and replace variable
	inmain = gxf.File(styledir+"main.template", "r")
	outmain = gxf.File(outputdir+python_file.title_ext, "w")
	main = gxf.Gxio(inmain, outmain)
	main.replace(replace)
	main.write_out(0755)
	inmain.close()
	outmain.close()
	
	#callbacks file writeout
	#detects if data exsist, if so, do not write header
	#in both cases, write new callbacks to callbacks file
	outcall = gxf.File(outputdir+callbacks_file.title_ext, "a+")
	outtext = outcall.read()
	#print "outtext: ",outtext=="", outtext;
	if outtext=="":
		incall = gxf.File(styledir+"callbacks.template", "r")
		call = gxf.Gxio(incall, outcall)
		call.replace(replace)
		call.set_text(call.get_text()+"\n"+get_callback_functions(outputdir,outcall.title_ext))
		call.write_out()
		incall.close()
	else:
		outcall.seek(os.stat(outcall.uri)[6])
		outcall.write_text("\n\n"+get_callback_functions(outputdir,outcall.title_ext))
	outcall.close()
	
	return "Job done. please open up the output\ndirectory to see the generated code."
#


def on_glade_file_change(widget):
	"""
	This function is executed when the glade file changes
	from the main window
	"""
	update_entry_feilds()
	pass
#

def on_directory_change(widget):
	pass
#

def on_info_button_clicked(widget="", event="", data=""):
	"""
	displays information about a style.
	"""
	style = tree.get_widget("code_style")
	style_name = style.get_active_text()
	message = ""
	if style_name == "Minimal":
		message = """Python Minimal code generation
	This plugin will hard-code callbacks in the main file.
	When a new function is defined in the glade file, it
	must get regenerated.
	"""
	if style_name == "Full":
		message = """Python Full code generation
	This plugin will include the required code to generate
	a dictionary, as well as append new callback to the
	callback file. You never have to run the code through
	gladex again.
	"""
	gxf.show_message(message)
	pass
#



def get_dictionary(indent):
	"""
	Returns dictionary code and callback functions.
	"""
	handarray = gxf.get_handlers()
	dictionarycode = ''
	for handler in handarray:
		dictionarycode = dictionarycode +  '\"' + handler + '\" : lambda w, d="": self.calls(\"' + handler + '\",w,d)\n '+indent*2
	dictionarycode = dictionarycode.strip()
	dictionarycode = dictionarycode.split('\n')
	dictionarycode = string.join(dictionarycode, ',\n\t')
	dictionarycode = indent*2+'# Dictionary autogenerated by gladex.\n'+indent*2+'dic = { ' + dictionarycode + ' }\n'+indent*2+'self.wTree.signal_autoconnect(dic)'
	return dictionarycode
#


def get_callback_functions(working_directory,callbacks_file_name):
	"""
	this function will return a string of all the functions that
	have not been defined yet in the callbacks file.
	"""
	#round about method to import from absloute path import
	sys.path.insert(0, working_directory)
	call_file_name = working_directory+callbacks_file_name
	callbacks = __import__(inspect.getmodulename(call_file_name))
	##
	a=""
	handarray = gxf.get_handlers()
	funcs = dir(callbacks)
	for c in handarray:
		# if function is already defined, then move on to the next one
		if c in funcs: continue
		else:
			a+="\ndef "+c+"(widget, data, wtree):\n\tprint \"function '"+c+"' not implemented\"\n\tpass\n"
	return a
#


