#!/usr/bin/env python
"""
    gladex - Generate source code from Glade XML files.

    Copyright (C) 2007 See AUTHORS list for copyright holders

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
sys.path.insert(0,"../..")
import gladex_functions as gxf
import gtk
import os
import re
import string
import inspect

main_widget = None
plugdir =os.path.split(__file__)[0]
tree = None
widgets = {}

def info():
	pass
info.name="Ruby Code Generator"
info.version=1
info.compatibility=0.4
info.description="""Ruby Code Generator
	If you are having trouble running the code,
	please make sure that you have ruby and 
	libglade2-ruby installed. Run this command
	sudo apt-get install ruby libglade2-ruby
"""
info.plugin_type="generator"

application_name=""

def widget():
	global main_widget, tree, parent_data, widgets,plugdir
	#parent_data = parent_input
		
	if main_widget == None:
	        gladefile = plugdir+'/ruby.glade'
	        tree = gtk.glade.XML(gladefile)
	        main_widget = tree.get_widget("main_widget")
	        dic ={"on_info_button_clicked":on_info_button_clicked}
	        tree.signal_autoconnect(dic)
	
	widgets["application_name"] = tree.get_widget("application_name_entry")	
	widgets["callbacks_file_name"] = tree.get_widget("callbacks_file_name_entry")
	widgets["ruby_file_name"] = tree.get_widget("ruby_file_name_entry")
	widgets["class_name"] = tree.get_widget("ruby_class_name_entry")
	widgets["indentation"] = tree.get_widget("indentation_chooser")
	widgets["indentation"].set_active(0)
	widgets["glade_file_entry"] = gxf.get_gladex_tree().get_widget("glade_file_chooser")
	
	update_entry_feilds()
	
	return main_widget

def update_entry_feilds():
	"""
	This function is executed when the fields of the main widget
	need to be updated.
	"""
	glade_file_name = widgets["glade_file_entry"].get_filename()
	if glade_file_name == None:
		glade_file_name="YourGladeFile"

		application_name="Application Name"
		widgets["application_name"].set_text(application_name)
	else:
		glade_file_name = glade_file_name[glade_file_name.rfind("/")+1:-6]

		application_name = glade_file_name[0].upper()+glade_file_name[1:]
		widgets["application_name"].set_text(application_name)
	
	callname = glade_file_name+"_callbacks.rb"
	widgets["callbacks_file_name"].set_text(make_valid(callname))
	
	classname = glade_file_name[0].upper()+glade_file_name[1:]+"Class"
	widgets["class_name"].set_text(make_valid(classname))
	
	rubyname = glade_file_name+".rb"
	widgets["ruby_file_name"].set_text(make_valid(rubyname))
#

def make_valid(text, extra=""):
	"""
	This function is executed when the plugin is loaded.
	Returns GTK widget of what to load.
	"""
	valid = "abcdefghijklmnopqrstuvwxyz"
	valid += valid.upper() + "._" + extra
	r=""
	for x in text:
		if x in valid:
			r+=x
	return r
#

def execute():
	valid = "abcdefghijklmnopqrstuvwxyz"
	valid += valid.upper()
	## check in all feilds
	gxt = gxf.get_gladex_tree()
	working_dir = gxt.get_widget("output_directory_chooser").get_filename()
	glade_file_name = gxt.get_widget("glade_file_chooser").get_filename()
	callbacks_file_name = tree.get_widget("callbacks_file_name_entry").get_text()
	ruby_file_name = tree.get_widget("ruby_file_name_entry").get_text()
	class_name = tree.get_widget("ruby_class_name_entry").get_text()
	indentation = tree.get_widget("indentation_chooser").get_active_text()
	
	
	glade_file = gxf.File(glade_file_name)
	callbacks_file = gxf.File(callbacks_file_name)
	ruby_file = gxf.File(ruby_file_name)
	
	## validate feilds
	if glade_file.ext != "glade":
		return "glade file must end in .glade"
	
	if callbacks_file.ext != "rb":
		return "callbacks file must end in .rb"
	
	if ruby_file.ext != "rb":
		return "Ruby file must end in .rb"
	
	for x in class_name:
		if x not in valid:
			return "there is a invalid character in class name"
	
	if indentation == "":
			return "please select a indentation type"
	if indentation == "tab":
		indent = "\t"
	elif indentation == "4 spaces":
		indent = "    "
	else:
		indent = "\t"
	
	## get callbacks functions
	callbacks = gxf.get_handlers()
	
	replace = {
		"{I}":indent,
		"{CLASS_NAME}":make_valid(class_name),
		"{APPLICATION_NAME}":make_valid(application_name),
		"{GLADE_FILE_TITLE_EXT}":glade_file.title_ext,
		"{GLADE_FILE_TITLE}":make_valid(glade_file.title,"1234567890_"),
		"{CALLBACKS_FILE_TITLE_EXT}":callbacks_file.title_ext,
		"{CALLBACKS_FILE_TITLE}":make_valid(callbacks_file.title,"1234567890_")
		}
	
	#input files
	#glade file copy
	##glade_file = gxf.File(glade_file_name, "r")
	##glade_file_out = gxf.File(working_dir+"/"+glade_file.title_ext, "w")
	##gfo = gxf.Gxio(glade_file, glade_file_out)
	##gfo.write_out()
	##glade_file.close()
	##glade_file_out.close()
	gxf.file_copy(glade_file_name, working_dir+"/")
	
	outputdir = working_dir+"/"
	#ruby file writeout useing template and replace variable
	inmain = gxf.File(plugdir+"/templates/main.template", "r")
	outmain = gxf.File(outputdir+ruby_file.title_ext, "w")
	main = gxf.Gxio(inmain, outmain)
	main.replace(replace)
	main.write_out(0755)
	inmain.close()
	outmain.close()
	
	#callbacks file writeout
	#detects if data exsist, if so, do not write header
	#in both cases, write new callbacks to callbacks file
	if not os.path.exists(outputdir+callbacks_file.title_ext):
		outcall = gxf.File(outputdir+callbacks_file.title_ext, "w+")
		incall = gxf.File(plugdir+"/templates/callbacks.template", "r")
		call = gxf.Gxio(incall, outcall)
		call.replace(replace)
		text = call.get_text()+"\n"
		text += get_callback_functions(outputdir,outcall.title_ext)
		call.set_text(text)
		call.write_out()
		incall.close()
	else:
		outcall = gxf.File(outputdir+callbacks_file.title_ext, "r")
		text = outcall.read_text()
		outcall.close()
		outcall = gxf.File(outputdir+callbacks_file.title_ext, "w")
		text += "\n"+get_callback_functions(outputdir,outcall.title_ext,text)
		outcall.write_text(text)
	outcall.close()
	
	return "Job done. please open up the output\ndirectory to see the generated code."

def on_glade_file_change(widget):
	update_entry_feilds()
	pass
#

def on_directory_change(widget):
	pass
#

def on_info_button_clicked(widget="", event="", data=""):
	style = tree.get_widget("code_style")
	
	gxf.show_message("info for style:"+str(style_name))
	pass
#



def get_callback_functions(odir, ofilename,textoption=None):
	"""
	Returns a string of all the function that are not in the callbacks
	file yet
	"""
	handlers = gxf.get_handlers()
	a = ''
	oldfunctions = get_old_callbacks(odir, ofilename,textoption)
	#print "########### old functions:",oldfunctions
	#print "########### handlers:",handlers
	for hand in handlers:
		if hand not in oldfunctions:
			a += "def "+hand+"()\n\tprint \"function '"+hand+"' not implemented\\n\"\nend\n"

	return a
#


def get_old_callbacks(odir, ofilename, textoption=None):
	"""
	this function uses regular expression to find all the functions in 
	a ruby file.
	"""
	if textoption==None:
		oldtext = open(odir+ofilename,'r').readlines()
	else:
		oldtext = textoption.split("\n")
	oldcalls=[]
	#print "oldtextf: ",odir+ofilename
	for line in oldtext:
		m = re.match("def",line)
		if(m):
			line = re.sub("def ","",line)
			line = line.strip()
			line = line[:line.find('(')]
			oldcalls.append(str(line).replace("\n",""))#m.group())
	return oldcalls
#

